use std::{env, sync::Arc};

use log::LevelFilter;
use simplelog::{ColorChoice, ConfigBuilder, TermLogger, TerminalMode};
use teloxide::prelude::*;

mod vk;

async fn on_inline_query(query: InlineQuery) -> anyhow::Result<()> {
    log::debug!("{:?}", query);

    let audios = vk::get_by_id(
        vk::search_audio(&query.query)
            .await?
            .items
            .iter()
            .map(|audio| format!("{}_{}", audio.owner_id, audio.id)),
    )
    .await?;
    log::debug!("{:?}", audios);

    Ok(())
}

async fn real_main() -> anyhow::Result<()> {
    if let Err(e) = dotenv::dotenv() {
        log::warn!("failed to load dotenv: {}", e);
    }

    let bot = Bot::new(env::var("TOKEN")?);

    async fn error_handler(e: anyhow::Error) {
        log::error!("{}", e);
    }

    Dispatcher::builder(
        bot,
        Update::filter_inline_query()
            .filter(|q: InlineQuery| !q.query.is_empty())
            .endpoint(on_inline_query),
    )
    .error_handler(Arc::new(error_handler))
    .build()
    .dispatch()
    .await;

    Ok(())
}

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    TermLogger::init(
        if cfg!(debug_assertions) {
            LevelFilter::Debug
        } else {
            LevelFilter::Info
        },
        ConfigBuilder::new()
            .add_filter_allow_str(module_path!())
            .build(),
        TerminalMode::Mixed,
        ColorChoice::Auto,
    )?;

    if let Err(e) = real_main().await {
        log::error!("{}", e);
    }

    Ok(())
}
