use serde::Deserialize;

mod request;
use request::{request, List};

#[derive(Debug, Deserialize)]
pub struct Audio {
    pub id: i32,
    pub owner_id: i32,
    pub artist: String,
    pub title: String,
}

#[derive(Debug, Deserialize)]
pub struct DetailedAudio {
    #[serde(flatten)]
    pub inner: Audio,
}

pub async fn search_audio<Q: AsRef<str>>(query: Q) -> anyhow::Result<List<Audio>> {
    request("audio.search", &[("q", query.as_ref())]).await
}

pub async fn get_by_id<I, S>(ids: I) -> anyhow::Result<List<DetailedAudio>>
where
    I: IntoIterator<Item = S>,
    S: Into<String>,
{
    request(
        "audio.getById",
        &[(
            "audios",
            ids.into_iter()
                .map(Into::into)
                .collect::<Vec<_>>()
                .join(","),
        )],
    )
    .await
}
