use lazy_static::lazy_static;
use serde::Deserialize;

lazy_static! {
    static ref CLIENT: reqwest::Client = reqwest::Client::new();
}

#[derive(Debug, Deserialize)]
struct Response<T> {
    response: Option<T>,
    error: Option<Error>,
}

#[derive(Debug, Deserialize)]
struct Error {
    error_code: i32,
    error_msg: String,
}

#[derive(Debug, Deserialize)]
pub struct List<T> {
    pub count: i32,
    pub items: Vec<T>,
}

pub async fn request<M: AsRef<str>, Q: serde::Serialize, R: serde::de::DeserializeOwned>(
    method: M,
    query: Q,
) -> anyhow::Result<R> {
    let response = CLIENT
        .request(
            reqwest::Method::GET,
            format!("https://api.vk.com/method/{}", method.as_ref()),
        )
        .query(&[
            ("access_token", std::env::var("VK_TOKEN")?),
            ("v", "5.131".to_string()),
        ])
        .query(&query)
        .send()
        .await?
        .json::<Response<R>>()
        .await?;

    if let Some(error) = response.error {
        return Err(anyhow::anyhow!("{}: {}", error.error_code, error.error_msg));
    }

    response
        .response
        .ok_or_else(|| anyhow::anyhow!("no response"))
}
